package l16_streams;

//	A team has more than one developer. Now we would like to know the aggregate
//	programming language skills for a given team. Stream#flatMap is perfect for
//	"flattening" collections:

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class FlatMapTest {

	record Developer(Set<String> languages) { }

	public static void main(String ... args) {
		Developer polyglot = new Developer(
			new HashSet<>(List.of("clojure scala groovy go python".split(" "))));
		System.out.println("polyglot: " + polyglot.languages());

		Developer busy = new Developer(
			new HashSet<>(List.of("python java javascript".split(" "))));
		System.out.println("busy: " + busy.languages());
		System.out.println();

		Set<String> teamLanguages =
			Stream.of(polyglot, busy)
			.map(Developer::languages)
			.flatMap(Collection::stream)
			.collect(Collectors.toCollection(TreeSet::new));
		System.out.println("team: " + teamLanguages);
	}
}
