package l16_streams;

import java.util.List;

class ParallelStreamTest {

	public static void main(String ... args) {

//	The parallel stream utilizes all available threads from the common
//	ForkJoinPool for executing the stream operations. The output may differ in
//	consecutive runs because the behavior which particular thread is actually
//	used is non-deterministic.

		List<String> stringList = List.of("one", "two", "three", "four", "five");

		stringList
			.parallelStream()
			.filter(string -> {
				System.out.format("\tfilter : %s [%s]%n", string, Thread.currentThread().getName());
				return true;
			})
			.map(string -> {
				System.out.format("\tmap    : %s [%s]%n", string, Thread.currentThread().getName());
				return string.toUpperCase();
			})
			.forEach(string ->
				System.out.format("forEach: %s [%s]%n", string, Thread.currentThread().getName()));
		System.out.println();

//	sort on a parallel stream uses the new Java 8 method Arrays.parallelSort()
//	under the hood. This method decides on the length of the array if sorting
//	will be performed sequentially or in parallel:

//	If the length of the specified array is less than the minimum granularity
//	(8x1024), then it is sorted using the appropriate Arrays.sort method.

		System.out.println();
		stringList
			.parallelStream()
			.filter(string -> {
				System.out.format("\tfilter : %s [%s]%n", string, Thread.currentThread().getName());
				return true;
			})
			.map(string -> {
				System.out.format("\tmap    : %s [%s]%n",
					string, Thread.currentThread().getName());
				return string.toUpperCase();
			})
			.sorted((string1, string2) -> {
				System.out.format("\tsort   : %s <> %s [%s]%n",
					string1, string2, Thread.currentThread().getName());
				return string1.compareTo(string2);
			})
//			.forEach(s -> System.out.format("forEach: %s [%s]%n",
			.forEachOrdered(string -> System.out.format("forEach: %s [%s]%n",
				string, Thread.currentThread().getName()));
		System.out.println();
	}
}

