package l16_streams;

import java.util.List;

class ReduceTest {

	public static void main(String ... args) {

		List<Person> persons = Person.createPersonList();

//	The reduction operation combines all elements of the stream into a single
//	result. Java 8 supports three different kind of reduce methods. The first
//	one reduces a stream of elements to exactly one element of the stream. Let's
//	see how we can use this method to determine the oldest person:

//	The reduce method accepts a BinaryOperator accumulator function. That's
//	actually a BiFunction where both operands share the same type, in that case
//	Person. The example	function compares both persons ages in order to return
//	the person with the maximum age.

		persons
			.stream()
			.reduce((person1, person2) ->
				person1.age() > person2.age() ? person1 : person2)
			.ifPresent(System.out::println); // Pamela

//	The second reduce method accepts both an identity value and a BinaryOperator
//	accumulator. This method can be utilized to construct a new Person with the
//	aggregated names and ages from all other persons in the stream:
		Person result =
			persons
			.stream()
			.reduce(new Person("", 0),
				(person1, person2) -> {
					person1.setAge(person1.age() + person2.age());
					person1.setName(person1.name() + " + " + person2.name());
					return person1;
			});
		System.out.println(result);
		System.out.println();

//	The third reduce method accepts three parameters: an identity value, a
//	BiFunction accumulator and a combiner function of type BinaryOperator. Since
//	the identity values type is not restricted to the Person type, we can
//	utilize this reduction to determine the sum of ages from all persons:

		int ageSumPar = persons
//			.stream()
			.parallelStream()
			.reduce(0,
				(sum, person) -> {
					System.out.format("\taccumulator: sum  = %3d; person = %9s [%s]%n",
						sum, person.name(), Thread.currentThread().getName());
					return sum + person.age();
				},
				(sum1, sum2) -> {
					System.out.format("\tcombiner   : sum1 = %3d; sum2   = %9d [%s]%n",
						sum1, sum2, Thread.currentThread().getName());
					return sum1 + sum2;
				});
		System.out.println(ageSumPar);
	}
}

