package l16_streams;

import java.util.List;
import java.util.stream.IntStream;

//	find the sum of even numbers and at the same time find the count of those
//	even numbers by using an external count variable.

public class StatefulExample {
	private static int count = 0;

	public static void main (String[] args) {
		System.out.println("Stateful count:");
		for (int i = 0; i < 5; i++) {
			process1();
		}
		System.out.println("Fixed version:");
		for (int i = 0; i < 5; i++) {
			process2();
		}
	}

	private static void process1() {
		count = 0;
		//finding the sum of even numbers
		int sum = IntStream.range(1, 1000) // sequence [1, 2, ..., 999]
			.parallel()
			.filter(i -> {
				if (i % 2 == 1) return false;
				count++;	//	updating count hence making lambda stateful.
				return true;
				})
			.sum();
		System.out.printf("sum :%d  count:%d%n", sum, count);
	}

//	To fix above code, find the sum and count separately:
	private static void process2() {

		//finding the even numbers
		int[] even = IntStream.range(1, 1000)
			.parallel()
			.filter(i -> i % 2 == 0)
			.toArray(); // IntStream doesn't have toList()

		//finding sum
		int sum = IntStream.of(even)
			.parallel()
			.sum();
		System.out.printf("sum :%d  count:%d%n", sum, even.length);
	}
}
