package l16_streams;

import java.util.stream.IntStream;
import java.util.stream.Stream;

public class StreamExamples {

	public static void main (String[] args) {

		System.out.println("Specialized streams");
		Stream.of("string 1", "string 2", "string 3")
			.map(string -> string.substring(string.length() - 1))
			.mapToInt(Integer::parseInt)
			.max()
			.ifPresent(System.out::println); // 3
		System.out.println();

		IntStream.range(1, 4)
			.mapToObj(i -> "string " + i)
			.forEach(System.out::println);	//	string 1 string 2 string 3
		System.out.println();

		String[] strings = "d2 a2 b1 b3 c".split(" ");

		System.out.println("laziness");
		Stream.of(strings)
			.filter(string -> {
				System.out.println("\tfilter: " + string);
				return true;
				//	no terminal operation - nothing is printed
			});
		System.out.println();

		System.out.println("Add terminal operation ...");
		System.out.println("Notice operation chaining");
		Stream.of(strings)
			.filter(string -> {
				System.out.println("\tfilter: " + string);
				return true;
			})
			.forEach(string -> System.out.println("forEach: " + string));
		System.out.println();

		System.out.println("anyMatch ...");
		boolean answer = Stream.of(strings)
			.map(string -> {
				System.out.println("\tmap: " + string);
				return string.toUpperCase();
			})
			.anyMatch(string -> {
				System.out.println("anyMatch: " + string);	//	returns true at first match
				return string.startsWith("A");
			});
		System.out.println("The answer is " + answer);
		System.out.println();

		System.out.println("Not optimal ...");
		Stream.of(strings)
			.map(string -> {
				System.out.println("\tmap: " + string);
				return string.toUpperCase();
			})
			.filter(string -> {
				System.out.println("\tfilter: " + string);
				return string.startsWith("A");
			})
			.forEach(string -> System.out.println("forEach: " + string));
		System.out.println();

		System.out.println("Optimal ...");
		Stream.of(strings)
			.filter(string -> {
				System.out.println("\tfilter: " + string);
				return string.startsWith("a");
			})
			.map(string -> {
				System.out.println("\tmap: " + string);
				return string.toUpperCase();
			})
			.forEach(string -> System.out.println("forEach: " + string));
		System.out.println();

		//	Sorting is a special kind of operation. It is so called 'stateful'
		//	operation, since while sorting we have to remember the current state.
		//	sorted() is called 'horizontally' for the entire collection.

		System.out.println("Sorting ...");
		Stream.of(strings)
			.sorted((string1, string2) -> {
				System.out.printf("\tsort: %s; %s%n", string1, string2);
				return string1.compareTo(string2);
			})
			.filter(string -> {
				System.out.println("\tfilter: " + string);
				return string.startsWith("a");
			})
			.map(string -> {
				System.out.println("\tmap: " + string);
				return string.toUpperCase();
			})
			.forEach(string -> System.out.println("forEach: " + string));
		System.out.println();

		System.out.println("Sorting optimized ...");
		Stream.of(strings)
			.filter(string -> {
				System.out.println("\tfilter: " + string);
				return string.startsWith("a");
			})
			.map(string -> {
				System.out.println("\tmap: " + string);
				return string.toUpperCase();
			})
			.sorted((string1, string2) -> {	//	not called for 1 element
				System.out.printf("\tsort: %s; %s%n", string1, string2);
				return string1.compareTo(string2);
			})
			.forEach(string -> System.out.println("forEach: " + string));
	}
}

