package l16_streams;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.function.Function;
import java.util.stream.Collectors;

public class StreamToCollection {
	public static void main(String[] args) {
		List<Person> personList = Person.createPersonList();

		System.out.println(nameToAgeMap(personList));
		System.out.println(ageToNameMap(personList));
		System.out.println(multiValueMap(personList));

		System.out.println(stringToCountMap(List.of("Java", "C++", "Java", "Python")));
	}

	private static Map<String, Integer> nameToAgeMap(List<Person> personList) {
		return personList.stream()
			.collect(Collectors.toMap(Person::name, Person::age));
	}

	private static Map<Integer, String> ageToNameMap(List<Person> personList) {
		return personList.stream()
			.collect(Collectors.toMap(
				Person::age, Person::name,
				(existing, replacement) -> existing // the first one stays
//				(existing, replacement) -> replacement // the last one stays
//				(name1, name2) -> name1 + " + " + name2 // both names
//				, TreeMap::new // TreeMap instead of HashMap (default)
			));
	}

	private static Map<Integer, List<Person>> multiValueMap(List<Person> personList) {
		return personList.stream()
			.collect(Collectors.groupingBy(
				Person::age,
				TreeMap::new,
				Collectors.toList()
			));
	}

	private static Map<String, Long> stringToCountMap(List<String> stringList) {
		return stringList.stream()
			.collect(Collectors.groupingBy(
				Function.identity(),
				TreeMap::new,
				Collectors.counting())
			);
	}
}
